#include <Arduino.h>
#include "shift_register.hpp"

void ShiftRegister::init(int dataPin, int sClkPin, int rClkPin, int oePin) {
  this->dataPin = dataPin;
  this->sClkPin = sClkPin;
  this->rClkPin = rClkPin;
  this->oePin = oePin;

  pinMode(dataPin, OUTPUT);
  pinMode(sClkPin, OUTPUT);
  pinMode(rClkPin, OUTPUT);
  pinMode(oePin, OUTPUT);
  // Disable the shift register
  digitalWrite(oePin, HIGH);
  shift_data(0b11111111);
  digitalWrite(oePin, LOW);
}

void ShiftRegister::shift_data(uint8_t data) {
    uint8_t mask = 0b00000001;
    // printf("Shifting in %d\r\n", data);
    while (mask != 0) {
        shift_in_bit(data & mask);
        mask <<= 1;
        // printf("Shifting, mask is: %d\r\n", mask);
    }
    pulse_register_clock();
}

inline void ShiftRegister::pulse_shift_clock() {
  // Digital write is slow enough (ms scale) to pulse the clocks (ns scale at
  // most)
  digitalWrite(sClkPin, HIGH);
  digitalWrite(sClkPin, LOW);
}

inline void ShiftRegister::pulse_register_clock() {
  // Digital write is slow enough (ms scale) to pulse the clocks (ns scale at
  // most)
  digitalWrite(rClkPin, HIGH);
  digitalWrite(rClkPin, LOW);
}

inline void ShiftRegister::shift_in_bit(bool bit) {
  digitalWrite(dataPin, bit);
  pulse_shift_clock();
}
